#ifndef s11n_QT_SERIALIZE_HPP_INCLUDED_INCLUDED
#define s11n_QT_SERIALIZE_HPP_INCLUDED_INCLUDED 1

#include <qsize.h>
#include <qpoint.h>
#include <qapplication.h>
#include <qclipboard.h>

#include <s11n.net/s11n/traits.hpp>
// #include <s11n.net/s11n/funxml_serializer.hpp>

namespace s11n {


        namespace qt {


                /**
                   An s11n proxy for QSize objects.
                */
                struct QSize_s11n
                {
                        /** Serialization operator. */
                        template <typename NodeT>
                        bool operator()( NodeT & dest, const QSize & src ) const
                        {
                                typedef s11n::node_traits<NodeT> NTR;
                                NTR::class_name( dest, "QSize" );
                                NTR::set( dest, "width", src.width() );
                                NTR::set( dest, "height", src.height() );
                                return true;
                                
                        }

                        /** Deserialization operator. */
                        template <typename NodeT>
                        bool operator()( const NodeT & src, QSize & dest ) const
                        {
                                typedef s11n::node_traits<NodeT> NTR;
                                dest.setWidth( NTR::get( src, "width", dest.width() ) );
                                dest.setHeight( NTR::get( src, "height", dest.height() ) );
                                return true;
                        }
                };

                /**
                   An s11n proxy for QPoint objects.
                */
                struct QPoint_s11n
                {
                        /** Serialization operator. */
                        template <typename NodeT>
                        bool operator()( NodeT & dest, const QPoint & src ) const
                        {
                                typedef s11n::node_traits<NodeT> NTR;
                                NTR::class_name( dest, "QPoint" );
                                NTR::set( dest, "x", src.x() );
                                NTR::set( dest, "y", src.y() );
                                return true;
                                
                        }

                        /** Deserialization operator. */
                        template <typename NodeT>
                        bool operator()( const NodeT & src, QPoint & dest ) const
                        {
                                typedef s11n::node_traits<NodeT> NTR;
                                dest.setX( NTR::get( src, "x", dest.x() ) );
                                dest.setY( NTR::get( src, "y", dest.y() ) );
                                return true;
                        }
                };

//                 template <typename NodeT>
//                 bool serializeNodeToClipboard( const NodeT & src )
//                 {
//                         typedef s11n::io::funxml_serializer<NodeT> SIO;
//                         std::ostringstream os;
//                         if( ! SIO().serialize( src, os ) ) return false;
//                         CERR << "sending to clipboard:\n"<<os.str()<<"\n";
//                         QApplication::clipboard()->setText( os.str(), QClipboard::Clipboard );
//                         return ! QApplication::clipboard()->text().isEmpty();
//                 }

//                 template <typename NodeT>
//                 NodeT * deserializeNodeFromClipboard()
//                 {
//                         typedef s11n::io::funxml_serializer<NodeT> SIO;

//                         QString txt = QApplication::clipboard()->text(QClipboard::Clipboard);
//                         if( txt.isEmpty() )
//                         {
//                                 CERR << "clipboard contains no text. :(\n";
//                                 return 0;
//                         }
//                         std::istringstream is( txt );
//                         return SIO().deserialize( is );
//                 }


//                 template <typename NodeT, typename SerializableT>
//                 bool serializableToClipboard( const SerializableT & src )
//                 {
//                         NodeT node;
//                         if( ! s11n::serialize( node, src ) ) return false;
//                         return serializeNodeToClipboard( node );
//                 }

//                 template <typename NodeT, typename SerializableT>
//                 SerializableT * serializableFromClipboard()
//                 {
                        
//                         QString txt = QApplication::clipboard()->text();
//                         if( txt.isEmpty() ) return 0;
//                         std::istringstream is( txt );
//                         typedef std::auto_ptr<NodeT> APN;
//                         APN node = APN( deserializeNodeFromClipboard<NodeT>() );
//                         return ( ! node.get() )
//                                 ? false
//                                 : s11n::deserialize<NodeT,SerializableT>( *node );
//                 }


        } // namespace qt

} // namespace s11n::qt


#define S11N_TYPE QSize
#define S11N_TYPE_NAME "QSize"
#define S11N_SERIALIZE_FUNCTOR s11n::qt::QSize_s11n
#include <s11n.net/s11n/reg_s11n_traits.hpp>

#define S11N_TYPE QPoint
#define S11N_TYPE_NAME "QPoint"
#define S11N_SERIALIZE_FUNCTOR s11n::qt::QPoint_s11n
#include <s11n.net/s11n/reg_s11n_traits.hpp>

// QString s11n untested:
#include <s11n.net/s11n/algo.hpp>
#define S11N_TYPE QString
#define S11N_TYPE_NAME "QString"
#define S11N_SERIALIZE_FUNCTOR s11n::streamable_type_serialization_proxy
#include <s11n.net/s11n/reg_s11n_traits.hpp>


#include <iostream>

/**
   Outputs sz in WIDTHxHEIGHT format.
*/
std::ostream & operator<<( std::ostream & os, const QSize & sz );

/**
   Reads in sz, expecting WIDTHxHEIGHT format.
*/
std::istream & operator>>( std::istream & is, QSize & sz );

/**
   Outputs pt in X,Y format.
*/
std::ostream & operator<<( std::ostream & os, const QPoint & pt );

/**
   Reads in pt, expecting X,Y format.
*/
std::istream & operator>>( std::istream & is, QPoint & pt );



#endif // s11n_QT_SERIALIZE_HPP_INCLUDED_INCLUDED
